package mi.hdm.mealPlan;

import mi.hdm.exceptions.InvalidMealPlanException;
import mi.hdm.recipes.NutritionCalculator;
import mi.hdm.recipes.NutritionTable;
import mi.hdm.recipes.Recipe;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.time.LocalDate;
import java.util.*;

/**
 * Allow the user to create a meal plan for the week. The user can check out his nutrition scores for the week.
 */
public class MealPlan {
    private static final Logger log = LogManager.getLogger(MealPlan.class);
    private final Map<LocalDate, Recipe> plan;

    public MealPlan(Map<LocalDate, Recipe> plan) {
        if (!plan.isEmpty()) {
            if (plan.keySet().stream().anyMatch(Objects::isNull))
                throw new InvalidMealPlanException("The specified map is invalid (null key)");
        }

        this.plan = plan;
    }

    public void addRecipeToMealPlan(Recipe recipe, LocalDate date) {
        if (date == null) {
            throw new NullPointerException("Date can not be null");
        }
        if (!isDateValid(date)) {
            throw new InvalidMealPlanException("You can only add recipes between today and in 7 days from now!");
        }
        plan.put(date, recipe);
    }

    public void clear(LocalDate date) {
        if (date == null) {
            throw new NullPointerException("Date can not be null");
        }

        if (plan.containsKey(date)) {
            log.info("Day {} cleared successfully.", date);
            plan.remove(date);
        } else {
            throw new NullPointerException("This date does not exist in the meal plan.");
        }
    }

    public void clear() {
        log.info("MealPlan cleared successfully.");
        plan.clear();
    }

    public Optional<Recipe> getRecipeForDay(LocalDate date) {
        return Optional.ofNullable(plan.get(date));
    }

    public Map<LocalDate, Recipe> getAllRecipesFromPlan() {
        return plan;
    }

    public void removePastRecipes() {
        final LocalDate now = LocalDate.now();
        List<LocalDate> toRemove = new ArrayList<>();
        for (LocalDate d : plan.keySet()) {
            if (d.isBefore(now)) toRemove.add(d);
        }
        toRemove.forEach(plan::remove);
        log.info("Removed {} recipes", toRemove.size());
    }

    public NutritionTable getNutritionTable() {
        return NutritionCalculator.calculateNutritionTable(this);
    }

    private boolean isDateValid(LocalDate date) {
        LocalDate invalid = LocalDate.now().plusDays(7);
        return !(date.isBefore(LocalDate.now()) || invalid.isBefore(date));
    }
}
